from hiv_workflow.lib.models.iemod_model import IEMODModel
from hiv_workflow.lib.utils.runtime import add_ingest_form_to_assets, add_post_channel_config_as_asset
from hiv_workflow.lib.utils.wrappers import generate_python_processing_wrapper


class EMOD_malaria(IEMODModel):

    def __init__(self, ingest_form_path=None, **kwargs):
        self.ingest_form_path = ingest_form_path
        if self.ingest_form_path is None:
            python_processing_wrapper, self.post_processing_config_file_setter = self._handle_python_processing()
            calibration_parameters = None
            site = None
        else:
            # ingest_info, site = get_ingest_information(ingest_filename=ingest_form_path)
            ingest_info = None
            site = None  # TODO: restore these in some fashion once a source is determined
            calibration_parameters = ingest_info['params']
            python_processing_wrapper, self.post_processing_config_file_setter = self._handle_python_processing(channels=ingest_info['channels'],
                                                                                                                reference_data=ingest_info['reference'],
                                                                                                                site_info=ingest_info['site_info'],
                                                                                                                pre_processing_path=kwargs['manifest'].pre_processing_path,
                                                                                                                in_processing_path=kwargs['manifest'].in_processing_path,
                                                                                                                post_processing_path=kwargs['manifest'].post_processing_path)
        super().__init__(python_processing_setter=python_processing_wrapper, calibration_parameters=calibration_parameters,
                         site=site, **kwargs)

    @staticmethod
    def _handle_python_processing(channels=None, reference_data=None, site_info=None,
                                  pre_processing_path=None, in_processing_path=None, post_processing_path=None):
        # generate channel config as an asset if requested and add post processing to match
        if channels and reference_data and site_info:
            def post_processing_config_file_setter(task):
                add_post_channel_config_as_asset(task=task, channels=channels, reference_data=reference_data, site_info=site_info)

            python_processing_setter = generate_python_processing_wrapper(pre=pre_processing_path,
                                                                          mid=in_processing_path,
                                                                          post=post_processing_path)
        else:
            post_processing_config_file_setter = None
            python_processing_setter = generate_python_processing_wrapper(pre=pre_processing_path,
                                                                          mid=in_processing_path)
        return python_processing_setter, post_processing_config_file_setter

    def initialize_task(self):
        task = super().initialize_task()

        # add the post processor config file if post-processing is requested
        if self.post_processing_config_file_setter:
            self.post_processing_config_file_setter(task=task)
        if self.ingest_form_path is not None:
            add_ingest_form_to_assets(task, self.ingest_form_path)
        return task

    def initialize_executable(self):
        import emod_malaria.bootstrap as bootstrap
        super().initialize_executable(bootstrapper=bootstrap)
